﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Itcast.JSMin.Helpers
{
    public class JavaScriptMinifier : IDisposable
    {
        // Fields
        private Encoding dstEncoding;
        private const int EOF = -1;
        private Encoding srcEncoding;
        private TextReader sReader;
        private TextWriter sWriter;
        private int theA;
        private int theB;
        private int theLookahead;
        private const bool TrimNewLines = false;

        // Methods
        public JavaScriptMinifier()
        {
            this.theLookahead = -1;
        }

        public JavaScriptMinifier(Encoding srcEncoding)
        {
            this.theLookahead = -1;
            this.srcEncoding = srcEncoding;
        }

        public JavaScriptMinifier(Encoding srcEncoding, Encoding dstEncoding)
        {
            this.theLookahead = -1;
            this.srcEncoding = srcEncoding;
            this.dstEncoding = dstEncoding;
        }

        private void action(int d)
        {
            if (d <= 1)
            {
                this.put(this.theA);
            }
            if (d <= 2)
            {
                this.theA = this.theB;
                if ((this.theA == 39) || (this.theA == 34))
                {
                    while (true)
                    {
                        this.put(this.theA);
                        this.theA = this.get();
                        if (this.theA == this.theB)
                        {
                            break;
                        }
                        if (this.theA <= 10)
                        {
                            throw new Exception(string.Format("Error: JSMIN unterminated string literal: {0}\n", this.theA));
                        }
                        if (this.theA == 92)
                        {
                            this.put(this.theA);
                            this.theA = this.get();
                        }
                    }
                }
            }
            if (d <= 3)
            {
                this.theB = this.next();
                if ((this.theB == 47) && (((((this.theA == 40) || (this.theA == 44)) || ((this.theA == 61) || (this.theA == 91))) || (((this.theA == 33) || (this.theA == 58)) || ((this.theA == 38) || (this.theA == 124)))) || (((this.theA == 63) || (this.theA == 123)) || (((this.theA == 125) || (this.theA == 59)) || (this.theA == 10)))))
                {
                    this.put(this.theA);
                    this.put(this.theB);
                    while (true)
                    {
                        this.theA = this.get();
                        if (this.theA == 47)
                        {
                            break;
                        }
                        if (this.theA == 92)
                        {
                            this.put(this.theA);
                            this.theA = this.get();
                        }
                        else if (this.theA <= 10)
                        {
                            throw new Exception(string.Format("Error: JSMIN unterminated Regular Expression literal : {0}.\n", this.theA));
                        }
                        this.put(this.theA);
                    }
                    this.theB = this.next();
                }
            }
        }

        public void Clear()
        {
            if (this.sReader != null)
            {
                try
                {
                    this.sReader.Close();
                    this.sReader.Dispose();
                }
                catch (Exception)
                {
                }
            }
            if (this.sWriter != null)
            {
                try
                {
                    this.sWriter.Close();
                    this.sWriter.Dispose();
                }
                catch (Exception)
                {
                }
            }
            this.theA = 0;
            this.theB = 0;
            this.theLookahead = -1;
        }

        public void Dispose()
        {
            this.Clear();
        }

        private string filterResultString(string srcStr, bool trimNewLines)
        {
            if (srcStr == null)
            {
                return srcStr;
            }
            if (srcStr.StartsWith("\n") || srcStr.StartsWith("\r\n"))
            {
                srcStr = srcStr.Remove(0, 1);
            }
            if (!trimNewLines)
            {
                return srcStr;
            }
            StringBuilder builder = new StringBuilder();
            using (TextReader reader = new StringReader(srcStr))
            {
                while (true)
                {
                    string str = reader.ReadLine();
                    if (str == null)
                    {
                        goto Label_0079;
                    }
                    if ((reader.Peek() > 0) && str.EndsWith("}"))
                    {
                        str = str + " ";
                    }
                    builder.Append(str);
                }
            }
        Label_0079:
            srcStr = builder.ToString();
            return srcStr;
        }

        private int get()
        {
            int theLookahead = this.theLookahead;
            this.theLookahead = -1;
            if (theLookahead == -1)
            {
                theLookahead = this.sReader.Read();
            }
            if (((theLookahead >= 32) || (theLookahead == 10)) || (theLookahead == -1))
            {
                return theLookahead;
            }
            if (theLookahead == 13)
            {
                return 10;
            }
            return 32;
        }

        private bool isAlphanum(int c)
        {
            if ((((c < 97) || (c > 122)) && ((c < 48) || (c > 57))) && (((c < 65) || (c > 90)) && (((c != 95) && (c != 36)) && (c != 92))))
            {
                return (c > 126);
            }
            return true;
        }

        private void jsmin()
        {
            this.theA = 10;
            this.action(3);
            while (this.theA != -1)
            {
                int num3;
                int theA = this.theA;
                if (theA == 10)
                {
                    switch (this.theB)
                    {
                        case 43:
                        case 45:
                        case 91:
                        case 123:
                        case 40:
                            goto Label_0085;

                        case 44:
                            goto Label_009D;

                        case 32:
                            goto Label_0091;
                    }
                    goto Label_009D;
                }
                if (theA != 32)
                {
                    goto Label_00C3;
                }
                if (this.isAlphanum(this.theB))
                {
                    this.action(1);
                }
                else
                {
                    this.action(2);
                }
                continue;
            Label_0085:
                this.action(1);
                continue;
            Label_0091:
                this.action(3);
                continue;
            Label_009D:
                if (this.isAlphanum(this.theB))
                {
                    this.action(1);
                }
                else
                {
                    this.action(2);
                }
                continue;
            Label_00C3:
                num3 = this.theB;
                if (num3 == 10)
                {
                    switch (this.theA)
                    {
                        case 93:
                        case 125:
                        case 39:
                        case 41:
                        case 43:
                        case 45:
                        case 34:
                            goto Label_0139;

                        case 40:
                        case 42:
                        case 44:
                            goto Label_0142;
                    }
                    goto Label_0142;
                }
                if (num3 != 32)
                {
                    goto Label_0162;
                }
                if (this.isAlphanum(this.theA))
                {
                    this.action(1);
                }
                else
                {
                    this.action(3);
                }
                continue;
            Label_0139:
                this.action(1);
                continue;
            Label_0142:
                if (this.isAlphanum(this.theA))
                {
                    this.action(1);
                }
                else
                {
                    this.action(3);
                }
                continue;
            Label_0162:
                this.action(1);
            }
        }


        public string MinifyString(string srcStr)
        {
            return this.MinifyString(srcStr, false);
        }

        public string MinifyString(string srcStr, bool trimNewLines)
        {
            if (srcStr == null)
            {
                return null;
            }
            this.Clear();
            this.sReader = new StringReader(srcStr);
            this.sWriter = new StringWriter();
            this.jsmin();
            this.sWriter.Flush();
            string str = this.filterResultString(this.sWriter.ToString(), trimNewLines);
            this.sWriter.Close();
            this.sWriter.Dispose();
            this.sWriter = null;
            this.sReader.Close();
            this.sReader.Dispose();
            this.sReader = null;
            return str;
        }

        private int next()
        {
            int num = this.get();
            if (num == 47)
            {
                switch (this.peek())
                {
                    case 42:
                        int num3;
                        this.get();
                        do
                        {
                            num3 = this.get();
                            if (num3 == -1)
                            {
                                throw new Exception("Error: JSMIN Unterminated comment.\n");
                            }
                        }
                        while ((num3 != 42) || (this.peek() != 47));
                        this.get();
                        return 32;

                    case 47:
                        do
                        {
                            num = this.get();
                        }
                        while (num > 10);
                        break;
                }
            }
            return num;
        }

        private int peek()
        {
            this.theLookahead = this.get();
            return this.theLookahead;
        }

        private void put(int c)
        {
            this.sWriter.Write((char)c);
        }
    }


}
